import { ownKeys } from './reflect';
export const hasSameOwnKeys = (a, b) => {
    const aKeys = ownKeys(a);
    const bKeys = ownKeys(b);
    if (aKeys.length !== bKeys.length) {
        return false;
    }
    for (let i = 0; i < aKeys.length; i++) {
        if (aKeys[i] !== bKeys[i]) {
            return false;
        }
    }
    return true;
};
export const returnFalse = () => false;
export const isObject = (value) => value !== null && typeof value === 'object';
const unfreezeCache = new WeakMap();
export const maybeUnfreeze = (value, kSource) => {
    if (!Object.isFrozen(value)) {
        return value;
    }
    const cached = unfreezeCache.get(value);
    if (cached !== undefined) {
        return cached;
    }
    let result;
    if (Array.isArray(value)) {
        result = Array.from(value);
    }
    else {
        const copy = {};
        const descriptors = Object.getOwnPropertyDescriptors(value);
        for (const descriptor of Object.values(descriptors)) {
            descriptor.configurable = true;
        }
        Object.defineProperties(copy, descriptors);
        result = copy;
    }
    Object.defineProperty(result, kSource, {
        configurable: false,
        enumerable: false,
        writable: false,
        value,
    });
    unfreezeCache.set(value, result);
    return result;
};
