"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.maybeUnfreeze = exports.isObject = exports.returnFalse = exports.hasSameOwnKeys = void 0;
const reflect_1 = require("./reflect");
const hasSameOwnKeys = (a, b) => {
    const aKeys = (0, reflect_1.ownKeys)(a);
    const bKeys = (0, reflect_1.ownKeys)(b);
    if (aKeys.length !== bKeys.length) {
        return false;
    }
    for (let i = 0; i < aKeys.length; i++) {
        if (aKeys[i] !== bKeys[i]) {
            return false;
        }
    }
    return true;
};
exports.hasSameOwnKeys = hasSameOwnKeys;
const returnFalse = () => false;
exports.returnFalse = returnFalse;
const isObject = (value) => value !== null && typeof value === 'object';
exports.isObject = isObject;
const unfreezeCache = new WeakMap();
const maybeUnfreeze = (value, kSource) => {
    if (!Object.isFrozen(value)) {
        return value;
    }
    const cached = unfreezeCache.get(value);
    if (cached !== undefined) {
        return cached;
    }
    let result;
    if (Array.isArray(value)) {
        result = Array.from(value);
    }
    else {
        const copy = {};
        const descriptors = Object.getOwnPropertyDescriptors(value);
        for (const descriptor of Object.values(descriptors)) {
            descriptor.configurable = true;
        }
        Object.defineProperties(copy, descriptors);
        result = copy;
    }
    Object.defineProperty(result, kSource, {
        configurable: false,
        enumerable: false,
        writable: false,
        value,
    });
    unfreezeCache.set(value, result);
    return result;
};
exports.maybeUnfreeze = maybeUnfreeze;
